/*
 *    **************************************************
 *    *                                                *
 *    *                    BIGIO.C                     *
 *    *                                                *
 *    *          Extended Precision Calculator         *
 *    *                                                *
 *    *             Low Level I/O Routines             *
 *    *                                                *
 *    *              Version 2.11 03-27-88             *
 *    *                                                *
 *    *              Judson D. McClendon               *
 *    *              844 Sun Valley Road               *
 *    *              Birmingham, AL 35215              *
 *    *                 205-853-8440                   *
 *    *            Compuserve [74415,1003]             *
 *    *                                                *
 *    **************************************************
 */



/*
 *    **************************************************
 *    *                                                *
 *    *                   Includes                     *
 *    *                                                *
 *    **************************************************
 */

#include <conio.h>
#include <ctype.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bigcalc.h"
#include "biggvar.h"



/*
 *    **************************************************
 *    *                                                *
 *    *                   Constants                    *
 *    *                                                *
 *    **************************************************
 */

#define CLOCKFREQ       1193180L /* Timer frequency */
#define SPEAKERMODE     0xB6     /* Set timer for speaker */
#define TIMERMODEPORT   0x43     /* Timer mode port */
#define FREQPORT        0x42     /* Frequency-control port */
#define SPEAKERPORT     0x61     /* Speaker port */
#define SPEAKERON       0x03     /* Speaker-on bits */
#define FREQ            400      /* A frequency */
#define CFREQ  CLOCKFREQ / FREQ  /* Division frequency */
#define COLOR           0x07     /* Grey on black */





/*
 *    **************************************************
 *    *                                                *
 *    *            Source Local Variables              *
 *    *                                                *
 *    **************************************************
 */

static union REGS cpu;





/*
 *    **************************************************
 *    *                                                *
 *    *              Keyboard Routines                 *
 *    *                                                *
 *    **************************************************
 */


/*
 *    **************************************************
 *    *                                                *
 *    *         Get decoded character from kbd         *
 *    *                                                *
 *    **************************************************
 */

int GetChar(void)

{

   static int chr;

   chr = getch();
   if (chr == 0)
      chr = 1000 + getch();   /* non ASCII character */
   else
      if (isascii(chr) )
         chr = toupper(chr);

   return(chr);

}




/*
 *    **************************************************
 *    *                                                *
 *    *         Return character if key pressed        *
 *    *                                                *
 *    **************************************************
 */

int KeyPressed(void)

{

   if (kbhit())
      return(GetChar());
   else
      return(0);

}






/*
 *    **************************************************
 *    *                                                *
 *    *                Screen Routines                 *
 *    *                                                *
 *    **************************************************
 */


/*
 *    **************************************************
 *    *                                                *
 *    *                Clear the Screen                *
 *    *                                                *
 *    **************************************************
 */

void ScrClr(void)

{

   CurPos(1, 1);
   EraEop();

}




/*
 *    **************************************************
 *    *                                                *
 *    *              Erase to End of Line              *
 *    *                                                *
 *    **************************************************
 */

void EraEol(void)

{

   static int col, row;

   CurGet(&row, &col);
   WNChar(' ', (81 - col));

}




/*
 *    **************************************************
 *    *                                                *
 *    *              Erase to End of Page              *
 *    *                                                *
 *    **************************************************
 */

void EraEop(void)

{

   static int row, col;

   CurGet(&row, &col);
   WNChar(' ', (80 * (25 - row) + 81 - col));

}




/*
 *    **************************************************
 *    *                                                *
 *    *         Position Cursor at row, column         *
 *    *                                                *
 *    **************************************************
 */

void CurPos(int row, int col)

{

   cpu.h.ah = 2;                 /* Set cursor position */
   cpu.h.dh = (char) row - 1;
   cpu.h.dl = (char) col - 1;
   cpu.h.bh = 0;
   int86(16, &cpu, &cpu);

}




/*
 *    **************************************************
 *    *                                                *
 *    *            Get Cursor row, column              *
 *    *                                                *
 *    **************************************************
 */

void CurGet(int *row, int *col)

{

   cpu.h.ah = 3;
   cpu.h.bh = 0;
   int86(16, &cpu, &cpu);
   *row = cpu.h.dh + 1;
   *col = cpu.h.dl + 1;

}




/*
 *    **************************************************
 *    *                                                *
 *    *     Write Character count times to Screen      *
 *    *                                                *
 *    **************************************************
 */

void WNChar(int chr, int count)

{

   cpu.h.ah = 9;
   cpu.h.al = (char) chr;
   cpu.h.bl = COLOR;
   cpu.h.bh = 0;
   cpu.x.cx = count;
   int86(16, &cpu, &cpu);

}




/*
 *    **************************************************
 *    *                                                *
 *    *            Write Character to Screen           *
 *    *                                                *
 *    **************************************************
 */

void WChar(int chr)

{

   putch(chr);

}




/*
 *    **************************************************
 *    *                                                *
 *    *             Write String to Screen             *
 *    *                                                *
 *    **************************************************
 */

void WString(char *str)

{

   cputs(str);

}




/*
 *    **************************************************
 *    *                                                *
 *    *             Write Integer to Screen            *
 *    *                                                *
 *    **************************************************
 */

void WInteger(long integer)

{

   static long order;

   if (integer) {
      if (integer < 0L) {
         WChar('-');
         integer = - integer;
         }
      order = 1000000L;
      while (order > integer) {
         order /= 10L;
         }
      while (order) {
         WChar((int)((integer / order) + (long)'0'));
         integer %= order;
         order /= 10L;
         }
      }
   else
      WChar('0');

}




/*
 *    **************************************************
 *    *                                                *
 *    *       Write String at row, col to Screen       *
 *    *                                                *
 *    **************************************************
 */

void WriteAt(int row, int col, char *str)

{

   CurPos(row, col);
   WString(str);

}




/*
 *    **************************************************
 *    *                                                *
 *    *     Display Message centered on row line       *
 *    *                                                *
 *    **************************************************
 */

void WriteCenter(int row, char *msg)

{

   CurPos(row, 1);
   EraEol();

   CurPos(row, ((80 - strlen(msg)) / 2));
   WString(msg);

}




/*
 *    **************************************************
 *    *                                                *
 *    *      Display Message centered on 25th line     *
 *    *                                                *
 *    **************************************************
 */

void Message(char *msg)

{

   WriteCenter(25, msg);

}




/*
 *    **************************************************
 *    *                                                *
 *    *      Write Message and Wait for keystroke      *
 *    *                                                *
 *    **************************************************
 */

void MessageWait(char *msg)

{

   static char tmsg[81];

   strcpy(tmsg, msg);
   if (*tmsg)
      strcat(tmsg, "  ");
   strcat(tmsg, "(Press a key to continue)");

   Message(tmsg);
   GetChar();

}




/*
 *    **************************************************
 *    *                                                *
 *    *      Write Message and Prompt for Escape       *
 *    *                                                *
 *    **************************************************
 */

void MessageEsc(char *msg)

{

   static char tmsg[81];

   strcpy(tmsg, msg);
   if (*tmsg)
      strcat(tmsg, "... ");
   strcat(tmsg, "(Press Esc to abort)");

   Message(tmsg);

}






/*
 *    **************************************************
 *    *                                                *
 *    *         Number Entry Support Routines          *
 *    *                                                *
 *    **************************************************
 */


/*
 *    **************************************************
 *    *                                                *
 *    *         Display Character at row, col          *
 *    *                                                *
 *    **************************************************
 */

void DisplayChar(int *row, int *col, int chr)

{

   CurPos(*row, *col);
   WChar(chr);

   if (*col < MAXDISPCOL)     /* Find position for next character */
      (*col)++;
   else {
      *col = MINDISPCOL;
      (*row)++;
      }
   CurPos(*row, *col);

}




/*
 *    **************************************************
 *    *                                                *
 *    *               Backspace Character              *
 *    *                                                *
 *    **************************************************
 */

void BackSpace(int *row, int *col)

{

   if (*col > MINDISPCOL)     /* find previous character position */
      (*col)--;
   else {
      *col = MAXDISPCOL;
      (*row)--;
      }

   CurPos(*row, *col);
   WChar(' ');
   CurPos(*row, *col);

}




/*
 *    **************************************************
 *    *                                                *
 *    *     Display Exponent Character at row, col     *
 *    *                                                *
 *    **************************************************
 */

void DisplayExpChar(int *row, int *col, int chr)

{

   CurPos(*row, *col);
   WChar(chr);

   if (*col < 80)             /* Find position for next character */
      (*col)++;
   else {
      *col = MINDISPCOL;
      (*row)++;
      }
   CurPos(*row, *col);

}




/*
 *    **************************************************
 *    *                                                *
 *    *          Backspace Exponent Character          *
 *    *                                                *
 *    **************************************************
 */

void BackSpaceExp(int *row, int *col)

{

   if (*col > MINDISPCOL)     /* find previous character position */
      (*col)--;
   else {
      *col = 80;
      (*row)--;
      }

   CurPos(*row, *col);
   WChar(' ');
   CurPos(*row, *col);

}




/*
 *    **************************************************
 *    *                                                *
 *    *          Display Exponent at row, col          *
 *    *                                                *
 *    **************************************************
 */

void DisplayExp(int *row, int *col, int exprow, int expcol, int expsign, long exponent)

{

   static long order;


   *row = exprow;
   *col = expcol;
   CurPos(exprow, expcol);          /* Locate to beginning of exponent */
   EraEol();

   if (expsign == '-')
      DisplayExpChar(row, col, '-');

   if (exponent) {                  /* Write exponent value */
      if (exponent < 0L) {
         DisplayExpChar(row, col, '-');
         exponent = - exponent;
         }
      order = 1000000L;
      while (order > exponent)
         order /= 10L;
      while (order) {
         DisplayExpChar(row, col, (int)((exponent / order) + (long) '0'));
         exponent %= order;
         order /= 10L;
         }
      }

}






/*
 *    **************************************************
 *    *                                                *
 *    *               Printer Routines                 *
 *    *                                                *
 *    **************************************************
 */


/*
 *    **************************************************
 *    *                                                *
 *    *           Print Character on Printer           *
 *    *                                                *
 *    **************************************************
 */

void PChar(int chr)

{

   if (!printtoscreen)
      fputc(chr, printfile);
   else
      WChar(chr);

}




/*
 *    **************************************************
 *    *                                                *
 *    *            Print String on Printer             *
 *    *                                                *
 *    **************************************************
 */

void PString(char *str)

{

   if (!printtoscreen)
      fputs(str, printfile);
   else
      cputs(str);

}




/*
 *    **************************************************
 *    *                                                *
 *    *            Print Integer on Printer            *
 *    *                                                *
 *    **************************************************
 */

void PInteger(long integer)

{

   static long order;

   if (integer) {
      if (integer < 0L) {
         PChar('-');
         integer = - integer;
         }
      order = 1000000L;
      while (order > integer) {
         order /= 10L;
         }
      while (order) {
         PChar((int)((integer / order) + (long)'0'));
         integer %= order;
         order /= 10L;
         }
      }
   else
      PChar('0');

}




/*
 *    **************************************************
 *    *                                                *
 *    *              New Line on Printer               *
 *    *                                                *
 *    **************************************************
 */

void NewLine(void)

{

   PString("\r\n");

}




/*
 *    **************************************************
 *    *                                                *
 *    *              New Page on Printer               *
 *    *                                                *
 *    **************************************************
 */

void NewPage(void)

{

   PString("\r\f");

}






/*
 *    **************************************************
 *    *                                                *
 *    *          Miscellaneous I/O Routines            *
 *    *                                                *
 *    **************************************************
 */


/*
 *    **************************************************
 *    *                                                *
 *    *                 Beep speaker                   *
 *    *                                                *
 *    **************************************************
 */

void Beep(void)

{

   static long endtime;
   static unsigned char saveport;

   outp(TIMERMODEPORT, SPEAKERMODE);
   saveport = (unsigned char) inp(SPEAKERPORT);

   outp(FREQPORT, CFREQ & 0xFF);          /* Start tone */
   outp(FREQPORT, CFREQ >> 8);
   outp(SPEAKERPORT, saveport | SPEAKERON);

   endtime = TimerTicks() + 2L;           /* Pause 2 clock ticks */
   while (TimerTicks() < endtime)
      ;

   outp(SPEAKERPORT, saveport);           /* Stop tone */

}




/*
 *    **************************************************
 *    *                                                *
 *    *         Fetch Timer Ticks since Midnight       *
 *    *                                                *
 *    **************************************************
 */

long TimerTicks(void)

{

   static long ticks;

   cpu.h.ah = 0;
   int86(0x1A, &cpu, &cpu);

   ticks = (((long) cpu.x.cx) << 16) + ((long) cpu.x.dx);
   return(ticks);

}
